@@ -177,10 +177,14 @@ ephy_bookmark_finalize (GObject *object)
 
   g_free (self->url);
   g_free (self->title);
-  g_free (self->id);
 
   g_sequence_free (self->tags);
 
+  g_free (self->id);
+  g_free (self->type);
+  g_free (self->parent_id);
+  g_free (self->parent_name);
+
   G_OBJECT_CLASS (ephy_bookmark_parent_class)->finalize (object);
 }
 
@@ -195,8 +199,7 @@ ephy_bookmark_class_init (EphyBookmarkClass *klass)
 
   obj_properties[PROP_TIME_ADDED] =
     g_param_spec_int64 ("time-added",
-                        "Time added",
-                        "The bookmark's creation time",
+                        NULL, NULL,
                         0,
                         G_MAXINT64,
                         0,
@@ -204,56 +207,48 @@ ephy_bookmark_class_init (EphyBookmarkClass *klass)
 
   obj_properties[PROP_ID] =
     g_param_spec_string ("id",
-                         "Id",
-                         "The bookmark's id",
+                         NULL, NULL,
                          "Default bookmark id",
                          G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_STRINGS);
 
   obj_properties[PROP_TITLE] =
     g_param_spec_string ("title",
-                         "Title",
-                         "The bookmark's title",
+                         NULL, NULL,
                          "Default bookmark title",
                          G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_STRINGS);
 
   obj_properties[PROP_BMK_URI] =
     g_param_spec_string ("bmkUri",
-                         "URI",
-                         "The bookmark's URI",
+                         NULL, NULL,
                          "about:overview",
                          G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_STRINGS);
 
   obj_properties[PROP_TAGS] =
     g_param_spec_pointer ("tags",
-                          "Tags",
-                          "The bookmark's tags",
+                          NULL, NULL,
                           G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_STRINGS);
 
   obj_properties[PROP_TYPE] =
     g_param_spec_string ("type",
-                         "Type",
-                         "Of type bookmark",
+                         NULL, NULL,
                          "default",
                          G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_STRINGS);
 
   obj_properties[PROP_PARENT_ID] =
     g_param_spec_string ("parentid",
-                         "ParentID",
-                         "The parent's id",
+                         NULL, NULL,
                          "default",
                          G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_STRINGS);
 
   obj_properties[PROP_PARENT_NAME] =
     g_param_spec_string ("parentName",
-                         "ParentName",
-                         "The parent's name",
+                         NULL, NULL,
                          "default",
                          G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_STRINGS);
 
   obj_properties[PROP_LOAD_IN_SIDEBAR] =
     g_param_spec_boolean ("loadInSidebar",
-                          "LoadInSiderbar",
-                          "Load in sidebar",
+                          NULL, NULL,
                           TRUE,
                           G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_STRINGS);
 
@@ -474,8 +469,8 @@ ephy_bookmark_bookmarks_compare_func (EphyBookmark *bookmark1,
   const char *title1;
   const char *title2;
   int title_result;
-  const char *id1;
-  const char *id2;
+  const char *url1;
+  const char *url2;
 
   g_assert (EPHY_IS_BOOKMARK (bookmark1));
   g_assert (EPHY_IS_BOOKMARK (bookmark2));
@@ -491,10 +486,10 @@ ephy_bookmark_bookmarks_compare_func (EphyBookmark *bookmark1,
   if (title_result != 0)
     return title_result;
 
-  id1 = ephy_bookmark_get_id (bookmark1);
-  id2 = ephy_bookmark_get_id (bookmark2);
+  url1 = ephy_bookmark_get_url (bookmark1);
+  url2 = ephy_bookmark_get_url (bookmark2);
 
-  return g_strcmp0 (id1, id2);
+  return g_strcmp0 (url1, url2);
 }
 
 int
