@@ -100,7 +100,9 @@ static const BindAction window_actions[] = {
 
   { EPHY_PREFS_LOCKDOWN_PRINTING, "print", "enabled" },
 
-  { EPHY_PREFS_LOCKDOWN_SAVE_TO_DISK, "save-as", "enabled" }
+  { EPHY_PREFS_LOCKDOWN_SAVE_TO_DISK, "save-as", "enabled" },
+
+  { EPHY_PREFS_LOCKDOWN_SAVE_TO_DISK, "screenshot", "enabled" }
 };
 
 static const BindAction popup_actions[] = {
@@ -111,10 +113,6 @@ static const BindAction popup_actions[] = {
   { EPHY_PREFS_LOCKDOWN_FULLSCREEN, "open-link-in-new-window", "enabled" }
 };
 
-static const BindAction tab_actions[] = {
-  { EPHY_PREFS_LOCKDOWN_FULLSCREEN, "detach", "enabled" }
-};
-
 static const BindAction toolbar_actions[] = {
   { EPHY_PREFS_LOCKDOWN_HISTORY, "navigation-back", "enabled" },
   { EPHY_PREFS_LOCKDOWN_HISTORY, "navigation-forward", "enabled" }
@@ -219,29 +217,19 @@ window_added_cb (GtkApplication *application,
                              action_group, app_mode_app_actions,
                              G_N_ELEMENTS (app_mode_app_actions));
 
-  action_group = gtk_widget_get_action_group (GTK_WIDGET (window),
-                                              "win");
+  action_group = ephy_window_get_action_group (EPHY_WINDOW (window), "win");
   bind_settings_and_actions (EPHY_SETTINGS_LOCKDOWN,
                              action_group,
                              window_actions,
                              G_N_ELEMENTS (window_actions));
 
-  action_group = gtk_widget_get_action_group (GTK_WIDGET (window),
-                                              "tab");
-  bind_settings_and_actions (EPHY_SETTINGS_LOCKDOWN,
-                             action_group,
-                             tab_actions,
-                             G_N_ELEMENTS (tab_actions));
-
-  action_group = gtk_widget_get_action_group (GTK_WIDGET (window),
-                                              "toolbar");
+  action_group = ephy_window_get_action_group (EPHY_WINDOW (window), "toolbar");
   bind_settings_and_actions (EPHY_SETTINGS_LOCKDOWN,
                              action_group,
                              toolbar_actions,
                              G_N_ELEMENTS (toolbar_actions));
 
-  action_group = gtk_widget_get_action_group (GTK_WIDGET (window),
-                                              "popup");
+  action_group = ephy_window_get_action_group (EPHY_WINDOW (window), "popup");
   bind_settings_and_actions (EPHY_SETTINGS_LOCKDOWN,
                              action_group, popup_actions,
                              G_N_ELEMENTS (popup_actions));
@@ -252,7 +240,8 @@ window_added_cb (GtkApplication *application,
   g_settings_bind_writable (settings, "picture-filename",
                             action, "enabled", FALSE);
 
-  if (mode != EPHY_EMBED_SHELL_MODE_APPLICATION) {
+  if (mode != EPHY_EMBED_SHELL_MODE_APPLICATION &&
+      mode != EPHY_EMBED_SHELL_MODE_AUTOMATION) {
     location_controller = ephy_window_get_location_controller (EPHY_WINDOW (window));
     bind_location_controller (EPHY_SETTINGS_LOCKDOWN, location_controller);
   }
