@@ -30,13 +30,13 @@
 #include "ephy-embed-utils.h"
 #include "ephy-embed.h"
 #include "ephy-file-helpers.h"
-#include "ephy-gui.h"
 #include "ephy-link.h"
-#include "ephy-notebook.h"
+#include "ephy-tab-view.h"
 #include "ephy-prefs.h"
 #include "ephy-settings.h"
 #include "ephy-shell.h"
 #include "ephy-string.h"
+#include "ephy-tab-view.h"
 #include "ephy-window.h"
 
 #include <glib/gi18n.h>
@@ -45,12 +45,12 @@
 #include <libxml/xmlwriter.h>
 
 typedef struct {
-  EphyNotebook *notebook;
+  EphyTabView *tab_view;
   gint ref_count;
-} NotebookTracker;
+} TabViewTracker;
 
 typedef struct {
-  NotebookTracker *notebook_tracker;
+  TabViewTracker *tab_view_tracker;
   int position;
   char *url;
   WebKitWebViewSessionState *state;
@@ -61,7 +61,6 @@ struct _EphySession {
 
   GQueue *closed_tabs;
   guint save_source_id;
-  GCancellable *save_cancellable;
   guint closing : 1;
   guint dont_save : 1;
   guint loaded_page : 1;
@@ -77,7 +76,7 @@ enum {
 
 static GParamSpec *obj_properties[LAST_PROP];
 
-static gboolean ephy_session_save_idle_cb (EphySession *session);
+static void ephy_session_save_now (EphySession *session);
 
 G_DEFINE_TYPE (EphySession, ephy_session, G_TYPE_OBJECT)
 
@@ -132,35 +131,34 @@ load_changed_cb (WebKitWebView   *view,
 }
 
 static void
-notebook_tracker_set_notebook (NotebookTracker *tracker,
-                               EphyNotebook    *notebook)
+tab_view_tracker_set_tab_view (TabViewTracker *tracker,
+                               EphyTabView    *tab_view)
 {
-  if (tracker->notebook == notebook) {
+  if (tracker->tab_view == tab_view)
     return;
-  }
 
-  if (tracker->notebook) {
-    g_object_remove_weak_pointer (G_OBJECT (tracker->notebook), (gpointer *)&tracker->notebook);
-  }
-  tracker->notebook = notebook;
-  if (tracker->notebook) {
-    g_object_add_weak_pointer (G_OBJECT (tracker->notebook), (gpointer *)&tracker->notebook);
-  }
+  if (tracker->tab_view)
+    g_object_remove_weak_pointer (G_OBJECT (tracker->tab_view), (gpointer *)&tracker->tab_view);
+
+  tracker->tab_view = tab_view;
+
+  if (tracker->tab_view)
+    g_object_add_weak_pointer (G_OBJECT (tracker->tab_view), (gpointer *)&tracker->tab_view);
 }
 
-static NotebookTracker *
-notebook_tracker_new (EphyNotebook *notebook)
+static TabViewTracker *
+tab_view_tracker_new (EphyTabView *tab_view)
 {
-  NotebookTracker *tracker = g_new0 (NotebookTracker, 1);
+  TabViewTracker *tracker = g_new0 (TabViewTracker, 1);
 
   tracker->ref_count = 1;
-  notebook_tracker_set_notebook (tracker, notebook);
+  tab_view_tracker_set_tab_view (tracker, tab_view);
 
   return tracker;
 }
 
-static NotebookTracker *
-notebook_tracker_ref (NotebookTracker *tracker)
+static TabViewTracker *
+tab_view_tracker_ref (TabViewTracker *tracker)
 {
   g_atomic_int_inc (&tracker->ref_count);
 
@@ -168,58 +166,58 @@ notebook_tracker_ref (NotebookTracker *tracker)
 }
 
 static void
-notebook_tracker_unref (NotebookTracker *tracker)
+tab_view_tracker_unref (TabViewTracker *tracker)
 {
   if (!g_atomic_int_dec_and_test (&tracker->ref_count))
     return;
 
-  notebook_tracker_set_notebook (tracker, NULL);
+  tab_view_tracker_set_tab_view (tracker, NULL);
   g_free (tracker);
 }
 
-static EphyNotebook *
-closed_tab_get_notebook (ClosedTab *tab)
+static EphyTabView *
+closed_tab_get_tab_view (ClosedTab *tab)
 {
-  return tab->notebook_tracker->notebook;
+  return tab->tab_view_tracker->tab_view;
 }
 
 static int
-compare_func (ClosedTab    *iter,
-              EphyNotebook *notebook)
+compare_func (ClosedTab   *iter,
+              EphyTabView *tab_view)
 {
-  return GTK_NOTEBOOK (closed_tab_get_notebook (iter)) - GTK_NOTEBOOK (notebook);
+  return (gpointer)closed_tab_get_tab_view (iter) - (gpointer)tab_view;
 }
 
-static NotebookTracker *
-ephy_session_ref_or_create_notebook_tracker (EphySession  *session,
-                                             EphyNotebook *notebook)
+static TabViewTracker *
+ephy_session_ref_or_create_tab_view_tracker (EphySession *session,
+                                             EphyTabView *tab_view)
 {
-  GList *item = g_queue_find_custom (session->closed_tabs, notebook, (GCompareFunc)compare_func);
-  return item ? notebook_tracker_ref (((ClosedTab *)item->data)->notebook_tracker) : notebook_tracker_new (notebook);
+  GList *item = g_queue_find_custom (session->closed_tabs, tab_view, (GCompareFunc)compare_func);
+  return item ? tab_view_tracker_ref (((ClosedTab *)item->data)->tab_view_tracker) : tab_view_tracker_new (tab_view);
 }
 
 static void
 closed_tab_free (ClosedTab *tab)
 {
   g_free (tab->url);
-  notebook_tracker_unref (tab->notebook_tracker);
+  tab_view_tracker_unref (tab->tab_view_tracker);
   webkit_web_view_session_state_unref (tab->state);
 
   g_free (tab);
 }
 
 static ClosedTab *
-closed_tab_new (EphyWebView     *web_view,
-                int              position,
-                NotebookTracker *notebook_tracker)
+closed_tab_new (EphyWebView    *web_view,
+                int             position,
+                TabViewTracker *tab_view_tracker)
 {
   ClosedTab *tab = g_new0 (ClosedTab, 1);
 
   tab->url = g_strdup (ephy_web_view_get_address (web_view));
   tab->position = position;
   /* Takes the ownership of the tracker */
-  tab->notebook_tracker = notebook_tracker;
-  tab->state = webkit_web_view_get_session_state (WEBKIT_WEB_VIEW (web_view));
+  tab->tab_view_tracker = tab_view_tracker;
+  tab->state = ephy_embed_get_session_state (EPHY_GET_EMBED_FROM_EPHY_WEB_VIEW (web_view));
 
   return tab;
 }
@@ -233,7 +231,7 @@ ephy_session_undo_close_tab (EphySession *session)
   WebKitBackForwardListItem *item;
   ClosedTab *tab;
   EphyWindow *window;
-  EphyNotebook *notebook;
+  EphyTabView *tab_view;
   EphyNewTabFlags flags = EPHY_NEW_TAB_JUMP;
 
   g_assert (EPHY_IS_SESSION (session));
@@ -243,12 +241,12 @@ ephy_session_undo_close_tab (EphySession *session)
     return;
 
   LOG ("UNDO CLOSE TAB: %s", tab->url);
-  notebook = closed_tab_get_notebook (tab);
-  if (notebook) {
+  tab_view = closed_tab_get_tab_view (tab);
+  if (tab_view) {
     if (tab->position > 0) {
       /* Append in the n-th position. */
-      embed = EPHY_EMBED (gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook),
-                                                     tab->position - 1));
+      embed = EPHY_EMBED (ephy_tab_view_get_nth_page (tab_view,
+                                                      tab->position - 1));
       flags |= EPHY_NEW_TAB_APPEND_AFTER;
     } else {
       /* Just prepend in the first position. */
@@ -256,7 +254,7 @@ ephy_session_undo_close_tab (EphySession *session)
       flags |= EPHY_NEW_TAB_FIRST;
     }
 
-    window = EPHY_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (notebook)));
+    window = EPHY_WINDOW (gtk_widget_get_root (GTK_WIDGET (tab_view)));
     new_tab = ephy_shell_new_tab (ephy_shell_get_default (),
                                   window, embed,
                                   flags);
@@ -264,8 +262,8 @@ ephy_session_undo_close_tab (EphySession *session)
     window = ephy_window_new ();
     new_tab = ephy_shell_new_tab (ephy_shell_get_default (),
                                   window, NULL, flags);
-    notebook_tracker_set_notebook (tab->notebook_tracker,
-                                   EPHY_NOTEBOOK (ephy_window_get_notebook (window)));
+    tab_view_tracker_set_tab_view (tab->tab_view_tracker,
+                                   ephy_window_get_tab_view (window));
   }
 
   web_view = WEBKIT_WEB_VIEW (ephy_embed_get_web_view (new_tab));
@@ -279,7 +277,7 @@ ephy_session_undo_close_tab (EphySession *session)
   }
 
   gtk_widget_grab_focus (GTK_WIDGET (new_tab));
-  gtk_window_present_with_time (GTK_WINDOW (window), gtk_get_current_event_time ());
+  gtk_window_present (GTK_WINDOW (window));
 
   closed_tab_free (tab);
 
@@ -288,10 +286,10 @@ ephy_session_undo_close_tab (EphySession *session)
 }
 
 static void
-ephy_session_tab_closed (EphySession  *session,
-                         EphyNotebook *notebook,
-                         EphyEmbed    *embed,
-                         gint          position)
+ephy_session_tab_closed (EphySession *session,
+                         EphyTabView *tab_view,
+                         EphyEmbed   *embed,
+                         gint         position)
 {
   EphyWebView *view;
   WebKitWebView *wk_view;
@@ -301,12 +299,13 @@ ephy_session_tab_closed (EphySession  *session,
   wk_view = WEBKIT_WEB_VIEW (view);
 
   if (!webkit_web_view_can_go_back (wk_view) && !webkit_web_view_can_go_forward (wk_view) &&
-      (ephy_web_view_get_is_blank (view) || ephy_web_view_is_overview (view))) {
+      (ephy_web_view_get_is_blank (view) || ephy_web_view_is_newtab (view) ||
+       ephy_web_view_is_overview (view))) {
     return;
   }
 
   tab = closed_tab_new (view, position,
-                        ephy_session_ref_or_create_notebook_tracker (session, notebook));
+                        ephy_session_ref_or_create_tab_view_tracker (session, tab_view));
   g_queue_push_head (session->closed_tabs, tab);
 
   if (g_queue_get_length (session->closed_tabs) == 1)
@@ -325,33 +324,38 @@ ephy_session_get_can_undo_tab_closed (EphySession *session)
 }
 
 static void
-notebook_page_added_cb (GtkWidget   *notebook,
-                        EphyEmbed   *embed,
-                        guint        position,
-                        EphySession *session)
+tab_view_page_attached_cb (AdwTabView  *tab_view,
+                           AdwTabPage  *page,
+                           guint        position,
+                           EphySession *session)
 {
+  EphyEmbed *embed = EPHY_EMBED (adw_tab_page_get_child (page));
+
   g_signal_connect (ephy_embed_get_web_view (embed), "load-changed",
                     G_CALLBACK (load_changed_cb), session);
 }
 
 static void
-notebook_page_removed_cb (GtkWidget   *notebook,
-                          EphyEmbed   *embed,
-                          guint        position,
-                          EphySession *session)
+tab_view_page_detached_cb (AdwTabView  *tab_view,
+                           AdwTabPage  *page,
+                           gint         position,
+                           EphySession *session)
 {
+  EphyEmbed *embed = EPHY_EMBED (adw_tab_page_get_child (page));
+  EphyTabView *ephy_tab_view = EPHY_TAB_VIEW (g_object_get_data (G_OBJECT (tab_view), "ephy-tab-view"));
+
   ephy_session_save (session);
 
   g_signal_handlers_disconnect_by_func
     (ephy_embed_get_web_view (embed), G_CALLBACK (load_changed_cb),
     session);
 
-  ephy_session_tab_closed (session, EPHY_NOTEBOOK (notebook), embed, position);
+  ephy_session_tab_closed (session, ephy_tab_view, embed, position);
 }
 
 static void
-notebook_page_reordered_cb (GtkWidget   *notebook,
-                            GtkWidget   *tab,
+tab_view_page_reordered_cb (AdwTabView  *tab_view,
+                            AdwTabPage  *page,
                             guint        position,
                             EphySession *session)
 {
@@ -359,10 +363,9 @@ notebook_page_reordered_cb (GtkWidget   *notebook,
 }
 
 static void
-notebook_switch_page_cb (GtkNotebook *notebook,
-                         GtkWidget   *page,
-                         guint        page_num,
-                         EphySession *session)
+tab_view_notify_selected_page_cb (AdwTabView  *tab_view,
+                                  GParamSpec  *pspec,
+                                  EphySession *session)
 {
   ephy_session_save (session);
 }
@@ -386,8 +389,8 @@ window_added_cb (GtkApplication *application,
                  GtkWindow      *window,
                  EphySession    *session)
 {
-  GtkWidget *notebook;
   EphyWindow *ephy_window;
+  AdwTabView *tab_view;
 
   ephy_session_save (session);
 
@@ -396,30 +399,16 @@ window_added_cb (GtkApplication *application,
 
   ephy_window = EPHY_WINDOW (window);
 
-  notebook = ephy_window_get_notebook (ephy_window);
-  g_signal_connect (notebook, "page-added",
-                    G_CALLBACK (notebook_page_added_cb), session);
-  g_signal_connect (notebook, "page-removed",
-                    G_CALLBACK (notebook_page_removed_cb), session);
-  g_signal_connect (notebook, "page-reordered",
-                    G_CALLBACK (notebook_page_reordered_cb), session);
-  g_signal_connect_after (notebook, "switch-page",
-                          G_CALLBACK (notebook_switch_page_cb), session);
-
-  /* Set unique identifier as role, so that on restore, the WM can
-   * place the window on the right workspace
-   */
-
-  if (gtk_window_get_role (window) == NULL) {
-    /* I guess rand() is unique enough, otherwise we could use
-     * time + pid or something
-     */
-    char *role;
-
-    role = g_strdup_printf ("epiphany-window-%x", rand ());
-    gtk_window_set_role (window, role);
-    g_free (role);
-  }
+  tab_view = ephy_tab_view_get_tab_view (ephy_window_get_tab_view (ephy_window));
+  g_signal_connect_object (tab_view, "page-attached",
+                           G_CALLBACK (tab_view_page_attached_cb), session, 0);
+  g_signal_connect_object (tab_view, "page-detached",
+                           G_CALLBACK (tab_view_page_detached_cb), session, 0);
+  g_signal_connect_object (tab_view, "page-reordered",
+                           G_CALLBACK (tab_view_page_reordered_cb), session, 0);
+  g_signal_connect_object (tab_view, "notify::selected-page",
+                           G_CALLBACK (tab_view_notify_selected_page_cb), session,
+                           G_CONNECT_AFTER);
 }
 
 static void
@@ -492,8 +481,7 @@ ephy_session_class_init (EphySessionClass *class)
 
   obj_properties[PROP_CAN_UNDO_TAB_CLOSED] =
     g_param_spec_boolean ("can-undo-tab-closed",
-                          "Can undo tab close",
-                          "Session can undo a tab closure",
+                          NULL, NULL,
                           FALSE,
                           G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
@@ -505,8 +493,6 @@ ephy_session_class_init (EphySessionClass *class)
 void
 ephy_session_close (EphySession *session)
 {
-  EphyPrefsRestoreSessionPolicy policy;
-
   g_assert (EPHY_IS_SESSION (session));
 
   LOG ("ephy_session_close");
@@ -521,12 +507,7 @@ ephy_session_close (EphySession *session)
 
   session->closing = TRUE;
 
-  policy = g_settings_get_enum (EPHY_SETTINGS_MAIN, EPHY_PREFS_RESTORE_SESSION_POLICY);
-  if (policy == EPHY_PREFS_RESTORE_SESSION_POLICY_ALWAYS) {
-    ephy_session_save_idle_cb (session);
-  } else {
-    session_delete (session);
-  }
+  ephy_session_save_now (session);
 
   session->dont_save = TRUE;
 }
@@ -535,7 +516,6 @@ typedef struct {
   GdkRectangle geometry;
   gboolean is_maximized;
   gboolean is_fullscreen;
-  char *role;
 
   GList *tabs;
   gint active_tab;
@@ -563,7 +543,7 @@ typedef struct {
 static SessionTab *
 session_tab_new (EphyEmbed   *embed,
                  EphySession *session,
-                 GtkNotebook *notebook)
+                 EphyTabView *tab_view)
 {
   SessionTab *session_tab;
   const char *address;
@@ -591,8 +571,8 @@ session_tab_new (EphyEmbed   *embed,
                           !session->closing);
   session_tab->crashed = (error_page == EPHY_WEB_VIEW_ERROR_PAGE_CRASH ||
                           error_page == EPHY_WEB_VIEW_ERROR_PROCESS_CRASH);
-  session_tab->state = webkit_web_view_get_session_state (WEBKIT_WEB_VIEW (web_view));
-  session_tab->pinned = ephy_notebook_tab_is_pinned (EPHY_NOTEBOOK (notebook), embed);
+  session_tab->state = ephy_embed_get_session_state (EPHY_GET_EMBED_FROM_EPHY_WEB_VIEW (web_view));
+  session_tab->pinned = ephy_tab_view_get_is_pinned (tab_view, GTK_WIDGET (embed));
 
   return session_tab;
 }
@@ -613,7 +593,7 @@ session_window_new (EphyWindow  *window,
 {
   SessionWindow *session_window;
   GList *tabs, *l;
-  GtkNotebook *notebook;
+  EphyTabView *tab_view;
 
   tabs = ephy_embed_container_get_children (EPHY_EMBED_CONTAINER (window));
   /* Do not save an empty EphyWindow.
@@ -625,19 +605,18 @@ session_window_new (EphyWindow  *window,
 
   session_window = g_new0 (SessionWindow, 1);
   get_window_geometry (window, session_window);
-  session_window->role = g_strdup (gtk_window_get_role (GTK_WINDOW (window)));
-  notebook = GTK_NOTEBOOK (ephy_window_get_notebook (window));
+  tab_view = ephy_window_get_tab_view (window);
 
   for (l = tabs; l != NULL; l = l->next) {
     SessionTab *tab;
 
-    tab = session_tab_new (EPHY_EMBED (l->data), session, notebook);
+    tab = session_tab_new (EPHY_EMBED (l->data), session, tab_view);
     session_window->tabs = g_list_prepend (session_window->tabs, tab);
   }
   g_list_free (tabs);
   session_window->tabs = g_list_reverse (session_window->tabs);
 
-  session_window->active_tab = gtk_notebook_get_current_page (notebook);
+  session_window->active_tab = ephy_tab_view_get_selected_index (tab_view);
 
   return session_window;
 }
@@ -645,7 +624,6 @@ session_window_new (EphyWindow  *window,
 static void
 session_window_free (SessionWindow *session_window)
 {
-  g_free (session_window->role);
   g_list_free_full (session_window->tabs, (GDestroyNotify)session_tab_free);
 
   g_free (session_window);
@@ -803,6 +781,24 @@ write_ephy_window (xmlTextWriterPtr  writer,
 {
   GList *l;
   int ret;
+  EphyPrefsRestoreSessionPolicy policy;
+  int last_pinned_tab = -1;
+  gboolean only_pinned_tabs = FALSE;
+
+  policy = g_settings_get_enum (EPHY_SETTINGS_MAIN, EPHY_PREFS_RESTORE_SESSION_POLICY);
+  only_pinned_tabs = policy == EPHY_PREFS_RESTORE_SESSION_POLICY_CRASHED;
+
+  if (only_pinned_tabs) {
+    for (l = window->tabs; l != NULL; l = l->next, last_pinned_tab++) {
+      SessionTab *tab = (SessionTab *)l->data;
+
+      if (!tab->pinned)
+        break;
+    }
+
+    if (last_pinned_tab == -1)
+      return 0;
+  }
 
   ret = xmlTextWriterStartElement (writer, (xmlChar *)"window");
   if (ret < 0)
@@ -812,21 +808,20 @@ write_ephy_window (xmlTextWriterPtr  writer,
   if (ret < 0)
     return ret;
 
+  if (last_pinned_tab != -1 && window->active_tab >= last_pinned_tab)
+    window->active_tab = last_pinned_tab + 1;
+
   ret = xmlTextWriterWriteFormatAttribute (writer, (const xmlChar *)"active-tab", "%d",
                                            window->active_tab);
   if (ret < 0)
     return ret;
 
-  if (window->role != NULL) {
-    ret = xmlTextWriterWriteAttribute (writer,
-                                       (const xmlChar *)"role",
-                                       (const xmlChar *)window->role);
-    if (ret < 0)
-      return ret;
-  }
-
   for (l = window->tabs; l != NULL; l = l->next) {
     SessionTab *tab = (SessionTab *)l->data;
+
+    if (only_pinned_tabs && !tab->pinned)
+      break;
+
     ret = write_tab (writer, tab);
     if (ret < 0)
       break;
@@ -834,6 +829,19 @@ write_ephy_window (xmlTextWriterPtr  writer,
   if (ret < 0)
     return ret;
 
+  if (only_pinned_tabs && last_pinned_tab != -1) {
+    /* We are in EPHY_PREFS_RESTORE_SESSION_POLICY_NEVER with pinned tabs
+     * Create a new overview page after the pinned tab
+     */
+    SessionTab *new_session_tab = g_new0 (SessionTab, 1);
+
+    new_session_tab->url = g_strdup ("about:overview");
+    new_session_tab->title = g_strdup ("");
+
+    write_tab (writer, new_session_tab);
+    session_tab_free (new_session_tab);
+  }
+
   ret = xmlTextWriterEndElement (writer);       /* window */
   return ret;
 }
@@ -843,16 +851,23 @@ save_session_in_thread_finished_cb (GObject      *source_object,
                                     GAsyncResult *res,
                                     gpointer      user_data)
 {
+  g_object_unref (EPHY_SESSION (source_object));
   g_application_release (G_APPLICATION (ephy_shell_get_default ()));
+
+  /* FIXME: this is a workaround for https://gitlab.gnome.org/GNOME/glib/-/issues/1346.
+   * After this GLib issue is fixed, we should instead pass save_data_free() as the
+   * GDestroyNotify parameter to g_task_set_task_data().
+   */
+  save_data_free (g_task_get_task_data (G_TASK (res)));
 }
 
 static gboolean
-session_seems_sane (GList *windows)
+session_seems_reasonable (GList *windows)
 {
   for (GList *w = windows; w != NULL; w = w->next) {
     for (GList *t = ((SessionWindow *)w->data)->tabs; t != NULL; t = t->next) {
       const char *url = ((SessionTab *)t->data)->url;
-      SoupURI *uri;
+      g_autoptr (GUri) uri = NULL;
       gboolean sane = FALSE;
 
       /* NULL URLs are possible when an invalid URL is opened by JS.
@@ -871,13 +886,14 @@ session_seems_sane (GList *windows)
       if (g_str_has_prefix (url, "about:"))
         continue;
 
-      uri = soup_uri_new (url);
+      uri = g_uri_parse (url, G_URI_FLAGS_NONE, NULL);
       if (uri) {
-        if (uri->host != NULL ||
-            uri->scheme == SOUP_URI_SCHEME_DATA ||
-            uri->scheme == SOUP_URI_SCHEME_FILE)
+        if (g_uri_get_host (uri) != NULL ||
+            strcmp (g_uri_get_scheme (uri), "data") == 0 ||
+            strcmp (g_uri_get_scheme (uri), "file") == 0 ||
+            strcmp (g_uri_get_scheme (uri), "ephy-reader") == 0 ||
+            strcmp (g_uri_get_scheme (uri), "view-source") == 0)
           sane = TRUE;
-        soup_uri_free (uri);
       }
 
       if (!sane) {
@@ -943,7 +959,7 @@ out:
   if (writer)
     xmlFreeTextWriter (writer);
 
-  if (ret >= 0 && !g_cancellable_is_cancelled (cancellable)) {
+  if (ret >= 0) {
     GError *error = NULL;
     GFile *session_file;
 
@@ -952,11 +968,8 @@ out:
     if (!g_file_replace_contents (session_file,
                                   (const char *)buffer->content,
                                   buffer->use,
-                                  NULL, TRUE, 0, NULL,
-                                  cancellable, &error)) {
-      if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
-        g_warning ("Error saving session: %s", error->message);
-      }
+                                  NULL, TRUE, 0, NULL, NULL, &error)) {
+      g_warning ("Error saving session: %s", error->message);
       g_error_free (error);
     }
 
@@ -971,21 +984,21 @@ out:
 }
 
 static EphySession *
-ephy_session_save_idle_started (EphySession *session)
+ephy_session_save_timeout_started (EphySession *session)
 {
   g_application_hold (G_APPLICATION (ephy_shell_get_default ()));
   return g_object_ref (session);
 }
 
 static void
-ephy_session_save_idle_finished (EphySession *session)
+ephy_session_save_timeout_finished (EphySession *session)
 {
   g_application_release (G_APPLICATION (ephy_shell_get_default ()));
   g_object_unref (session);
 }
 
 static gboolean
-ephy_session_save_idle_cb (EphySession *session)
+ephy_session_save_timeout_cb (EphySession *session)
 {
   EphyShell *shell = ephy_shell_get_default ();
   SaveData *data;
@@ -993,6 +1006,9 @@ ephy_session_save_idle_cb (EphySession *session)
 
   session->save_source_id = 0;
 
+  if (!session->loaded_page)
+    return G_SOURCE_REMOVE;
+
   /* If we have never successfully loaded any page, or any web view has an
    * insane URL, then something has probably gone wrong inside WebKit. For
    * instance, if the web process is nonfunctional, the UI process could have
@@ -1001,17 +1017,11 @@ ephy_session_save_idle_cb (EphySession *session)
    * file with our new bogus state. Bug #768250.
    */
   data = save_data_new (session);
-  if (!session->loaded_page || !session_seems_sane (data->windows)) {
+  if (!session_seems_reasonable (data->windows)) {
     save_data_free (data);
     return G_SOURCE_REMOVE;
   }
 
-  if (session->save_cancellable) {
-    g_cancellable_cancel (session->save_cancellable);
-    g_object_unref (session->save_cancellable);
-    session->save_cancellable = NULL;
-  }
-
   LOG ("ephy_sesion_save");
 
   if (ephy_shell_get_n_windows (shell) == 0) {
@@ -1021,10 +1031,10 @@ ephy_session_save_idle_cb (EphySession *session)
   }
 
   g_application_hold (G_APPLICATION (ephy_shell_get_default ()));
-  session->save_cancellable = g_cancellable_new ();
-  task = g_task_new (session, session->save_cancellable,
-                     save_session_in_thread_finished_cb, NULL);
-  g_task_set_task_data (task, data, (GDestroyNotify)save_data_free);
+  g_object_ref (session);
+
+  task = g_task_new (session, NULL, save_session_in_thread_finished_cb, NULL);
+  g_task_set_task_data (task, data, NULL);
   g_task_run_in_thread (task, save_session_sync);
   g_object_unref (task);
 
@@ -1034,8 +1044,6 @@ ephy_session_save_idle_cb (EphySession *session)
 void
 ephy_session_save (EphySession *session)
 {
-  EphyPrefsRestoreSessionPolicy policy;
-
   g_assert (EPHY_IS_SESSION (session));
 
   if (session->save_source_id)
@@ -1044,14 +1052,19 @@ ephy_session_save (EphySession *session)
   if (session->dont_save)
     return;
 
-  policy = g_settings_get_enum (EPHY_SETTINGS_MAIN, EPHY_PREFS_RESTORE_SESSION_POLICY);
-  if (policy == EPHY_PREFS_RESTORE_SESSION_POLICY_NEVER)
-    return;
-
+  /* Schedule the save to occur one second in the future to ensure we don't
+   * repeatedly write to disk when opening or closing many tabs at once.
+   */
   session->save_source_id = g_timeout_add_seconds_full (G_PRIORITY_DEFAULT_IDLE, 1,
-                                                        (GSourceFunc)ephy_session_save_idle_cb,
-                                                        ephy_session_save_idle_started (session),
-                                                        (GDestroyNotify)ephy_session_save_idle_finished);
+                                                        (GSourceFunc)ephy_session_save_timeout_cb,
+                                                        ephy_session_save_timeout_started (session),
+                                                        (GDestroyNotify)ephy_session_save_timeout_finished);
+}
+
+static void
+ephy_session_save_now (EphySession *session)
+{
+  ephy_session_save_timeout_cb (session);
 }
 
 static void
@@ -1075,9 +1088,6 @@ static void
 restore_geometry (GtkWindow    *window,
                   GdkRectangle *geometry)
 {
-  if (geometry->x >= 0 && geometry->y >= 0)
-    ephy_window_set_default_position (EPHY_WINDOW (window), geometry->x, geometry->y);
-
   if (geometry->width > 0 && geometry->height > 0)
     ephy_window_set_default_size (EPHY_WINDOW (window), geometry->width, geometry->height);
 }
@@ -1121,6 +1131,14 @@ session_parser_context_free (SessionParserContext *context)
   g_free (context);
 }
 
+static void
+window_destroyed (GtkWidget  *widget,
+                  GtkWidget **widget_pointer)
+{
+  if (widget_pointer)
+    *widget_pointer = NULL;
+}
+
 static void
 session_parse_window (SessionParserContext  *context,
                       const gchar          **names,
@@ -1137,7 +1155,7 @@ session_parse_window (SessionParserContext  *context,
   }
 
   context->window = ephy_window_new ();
-  context->destroy_id = g_signal_connect (context->window, "destroy", G_CALLBACK (gtk_widget_destroyed), &context->window);
+  context->destroy_id = g_signal_connect (context->window, "destroy", G_CALLBACK (window_destroyed), &context->window);
 
   for (i = 0; names[i]; i++) {
     gulong int_value;
@@ -1160,8 +1178,6 @@ session_parse_window (SessionParserContext  *context,
     } else if (strcmp (names[i], "is-fullscreen") == 0) {
       ephy_string_to_int (values[i], &int_value);
       is_fullscreen = int_value != 0;
-    } else if (strcmp (names[i], "role") == 0) {
-      gtk_window_set_role (GTK_WINDOW (context->window), values[i]);
     } else if (strcmp (names[i], "active-tab") == 0) {
       ephy_string_to_int (values[i], &int_value);
       context->active_tab = int_value;
@@ -1173,8 +1189,11 @@ session_parse_window (SessionParserContext  *context,
   if (is_maximized)
     gtk_window_maximize (GTK_WINDOW (context->window));
 
-  if (is_fullscreen)
+  if (is_fullscreen) {
+    /* Treat fullscreen on session restore same as fullscreen action */
+    ephy_window_show_fullscreen_header_bar (context->window);
     gtk_window_fullscreen (GTK_WINDOW (context->window));
+  }
 }
 
 static void
@@ -1182,7 +1201,7 @@ session_parse_embed (SessionParserContext  *context,
                      const gchar          **names,
                      const gchar          **values)
 {
-  GtkWidget *notebook;
+  AdwTabView *tab_view;
   const char *url = NULL;
   const char *title = NULL;
   const char *history = NULL;
@@ -1199,7 +1218,7 @@ session_parse_embed (SessionParserContext  *context,
     return;
   }
 
-  notebook = ephy_window_get_notebook (context->window);
+  tab_view = ephy_tab_view_get_tab_view (ephy_window_get_tab_view (context->window));
 
   for (i = 0; names[i]; i++) {
     if (strcmp (names[i], "url") == 0) {
@@ -1249,7 +1268,9 @@ session_parse_embed (SessionParserContext  *context,
                                      context->window, NULL, flags,
                                      0);
 
-    ephy_notebook_tab_set_pinned (EPHY_NOTEBOOK (notebook), GTK_WIDGET (embed), is_pin);
+    adw_tab_view_set_page_pinned (tab_view,
+                                  adw_tab_view_get_page (tab_view, GTK_WIDGET (embed)),
+                                  is_pin);
 
     web_view = ephy_embed_get_web_view (embed);
     if (history) {
@@ -1325,7 +1346,7 @@ session_end_element (GMarkupParseContext  *ctx,
   SessionParserContext *context = (SessionParserContext *)user_data;
 
   if (strcmp (element_name, "window") == 0) {
-    GtkWidget *notebook;
+    EphyTabView *tab_view;
     EphyEmbedShell *shell = ephy_embed_shell_get_default ();
 
     if (!context->window) {
@@ -1335,8 +1356,9 @@ session_end_element (GMarkupParseContext  *ctx,
       return;
     }
 
-    notebook = ephy_window_get_notebook (context->window);
-    gtk_notebook_set_current_page (GTK_NOTEBOOK (notebook), context->active_tab);
+    tab_view = ephy_window_get_tab_view (context->window);
+    if (context->active_tab < ephy_tab_view_get_n_pages (tab_view))
+      ephy_tab_view_select_nth_page (tab_view, context->active_tab);
 
     if (ephy_embed_shell_get_mode (ephy_embed_shell_get_default ()) != EPHY_EMBED_SHELL_MODE_TEST) {
       EphyEmbed *active_child;
@@ -1344,7 +1366,7 @@ session_end_element (GMarkupParseContext  *ctx,
       active_child = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (context->window));
       gtk_widget_grab_focus (GTK_WIDGET (active_child));
       ephy_window_update_entry_focus (context->window, ephy_embed_get_web_view (active_child));
-      gtk_widget_show (GTK_WIDGET (context->window));
+      gtk_widget_set_visible (GTK_WIDGET (context->window), TRUE);
     }
 
     ephy_embed_shell_restored_window (shell);
@@ -1730,7 +1752,6 @@ ephy_session_resume (EphySession         *session,
 {
   GTask *task;
   gboolean has_session_state;
-  EphyPrefsRestoreSessionPolicy policy;
   EphyShell *shell;
 
   LOG ("ephy_session_resume");
@@ -1739,20 +1760,13 @@ ephy_session_resume (EphySession         *session,
 
   has_session_state = session_state_file_exists (session);
 
-  policy = g_settings_get_enum (EPHY_SETTINGS_MAIN,
-                                EPHY_PREFS_RESTORE_SESSION_POLICY);
-
   shell = ephy_shell_get_default ();
 
   /* If we are auto-resuming, and we never want to
    * restore the session, clobber the session state
    * file.
    */
-  if (policy == EPHY_PREFS_RESTORE_SESSION_POLICY_NEVER)
-    session_delete (session);
-
-  if (has_session_state == FALSE ||
-      policy == EPHY_PREFS_RESTORE_SESSION_POLICY_NEVER) {
+  if (has_session_state == FALSE) {
     session_maybe_open_window (session, user_time);
   } else if (ephy_shell_get_n_windows (shell) == 0) {
     ephy_session_load (session, SESSION_STATE, user_time, cancellable,
@@ -1786,7 +1800,7 @@ ephy_session_clear (EphySession *session)
   shell = ephy_shell_get_default ();
   windows = g_list_copy (gtk_application_get_windows (GTK_APPLICATION (shell)));
   for (p = windows; p; p = p->next)
-    gtk_widget_destroy (GTK_WIDGET (p->data));
+    gtk_window_destroy (GTK_WINDOW (p->data));
   g_list_free (windows);
   g_queue_foreach (session->closed_tabs,
                    (GFunc)closed_tab_free, NULL);
