@@ -20,17 +20,15 @@
 
 #include "config.h"
 
-#include <dazzle.h>
+#include <adwaita.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
-#include <handy.h>
 #include <string.h>
 
 #define SECRET_API_SUBJECT_TO_CHANGE
 #include <libsecret/secret.h>
 
 #include "ephy-shell.h"
-#include "ephy-gui.h"
 #include "ephy-uri-helpers.h"
 #include "passwords-view.h"
 
@@ -67,15 +65,10 @@ ephy_passwords_view_dispose (GObject *object)
 static void
 clear_listbox (GtkWidget *listbox)
 {
-  GList *children, *iter;
+  GtkListBoxRow *row;
 
-  children = gtk_container_get_children (GTK_CONTAINER (listbox));
-
-  for (iter = children; iter; iter = g_list_next (iter)) {
-    gtk_widget_destroy (GTK_WIDGET (iter->data));
-  }
-
-  g_list_free (children);
+  while ((row = gtk_list_box_get_row_at_index (GTK_LIST_BOX (listbox), 0)))
+    gtk_list_box_remove (GTK_LIST_BOX (listbox), GTK_WIDGET (row));
 }
 
 static void
@@ -134,7 +127,7 @@ copy_password_clicked (GtkWidget *button,
   const char *password = user_data;
 
   if (password)
-    gtk_clipboard_set_text (gtk_widget_get_clipboard (GTK_WIDGET (button), GDK_SELECTION_CLIPBOARD), password, -1);
+    gdk_clipboard_set_text (gtk_widget_get_clipboard (GTK_WIDGET (button)), password);
 }
 
 static void
@@ -144,7 +137,7 @@ copy_username_clicked (GtkWidget *button,
   const char *username = user_data;
 
   if (username)
-    gtk_clipboard_set_text (gtk_widget_get_clipboard (GTK_WIDGET (button), GDK_SELECTION_CLIPBOARD), username, -1);
+    gdk_clipboard_set_text (gtk_widget_get_clipboard (GTK_WIDGET (button)), username);
 }
 
 static void
@@ -163,53 +156,40 @@ ephy_passwords_view_class_init (EphyPasswordsViewClass *klass)
 }
 
 static void
-confirmation_dialog_response_cb (GtkWidget         *dialog,
-                                 int                response,
-                                 EphyPasswordsView *self)
+confirmation_dialog_response_cb (EphyPasswordsView *self)
 {
-  gtk_widget_destroy (dialog);
+  ephy_password_manager_forget_all (self->manager);
 
-  if (response == GTK_RESPONSE_ACCEPT) {
-    ephy_password_manager_forget_all (self->manager);
+  clear_listbox (self->listbox);
+  ephy_data_view_set_has_data (EPHY_DATA_VIEW (self), FALSE);
 
-    clear_listbox (self->listbox);
-    ephy_data_view_set_has_data (EPHY_DATA_VIEW (self), FALSE);
-
-    g_list_free_full (self->records, g_object_unref);
-    self->records = NULL;
-  }
+  g_list_free_full (self->records, g_object_unref);
+  self->records = NULL;
 }
 
 static GtkWidget *
 confirmation_dialog_construct (EphyPasswordsView *self)
 {
   GtkWidget *dialog;
-  GtkWidget *button;
-  GtkWidget *window;
+  GtkRoot *window;
 
-  window = gtk_widget_get_toplevel (GTK_WIDGET (self));
+  window = gtk_widget_get_root (GTK_WIDGET (self));
 
-  dialog = gtk_message_dialog_new (GTK_WINDOW (window),
-                                   GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
-                                   GTK_MESSAGE_WARNING,
-                                   GTK_BUTTONS_CANCEL,
-                                   _("Delete All Passwords?"));
+  dialog = adw_message_dialog_new (GTK_WINDOW (window),
+                                   _("Delete All Passwords?"),
+                                   _("This will clear all locally stored passwords, and can not be undone."));
 
-  gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-                                            _("This will clear all locally stored passwords, and can not be undone."));
+  adw_message_dialog_add_responses (ADW_MESSAGE_DIALOG (dialog),
+                                    "cancel", _("_Cancel"),
+                                    "delete", _("_Delete"),
+                                    NULL);
+  adw_message_dialog_set_response_appearance (ADW_MESSAGE_DIALOG (dialog),
+                                              "delete",
+                                              ADW_RESPONSE_DESTRUCTIVE);
 
-  gtk_window_group_add_window (ephy_gui_ensure_window_group (GTK_WINDOW (window)),
-                               GTK_WINDOW (dialog));
-
-  button = gtk_button_new_with_mnemonic (_("_Delete"));
-  gtk_widget_show (button);
-  gtk_dialog_add_action_widget (GTK_DIALOG (dialog), button, GTK_RESPONSE_ACCEPT);
-
-  gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_CANCEL);
-
-  g_signal_connect (dialog, "response",
-                    G_CALLBACK (confirmation_dialog_response_cb),
-                    self);
+  g_signal_connect_swapped (dialog, "response::delete",
+                            G_CALLBACK (confirmation_dialog_response_cb),
+                            self);
 
   return dialog;
 }
@@ -230,7 +210,7 @@ forget_all (GSimpleAction *action,
                                (gpointer *)confirmation_dialog);
   }
 
-  gtk_widget_show (self->confirmation_dialog);
+  gtk_widget_set_visible (self->confirmation_dialog, TRUE);
 }
 
 static void
@@ -245,105 +225,66 @@ populate_model_cb (GList    *records,
     EphyPasswordRecord *record = EPHY_PASSWORD_RECORD (l->data);
     GtkWidget *row;
     GtkWidget *sub_row;
-    GtkWidget *separator;
     GtkWidget *button;
-    GtkWidget *image;
-    GtkWidget *entry;
     const char *text;
 
-    row = hdy_expander_row_new ();
+    row = adw_expander_row_new ();
     g_object_set_data (G_OBJECT (row), "record", record);
-    hdy_preferences_row_set_title (HDY_PREFERENCES_ROW (row), ephy_password_record_get_origin (record));
-    hdy_expander_row_set_subtitle (HDY_EXPANDER_ROW (row), ephy_password_record_get_username (record));
-    hdy_expander_row_set_show_enable_switch (HDY_EXPANDER_ROW (row), FALSE);
+    adw_preferences_row_set_title (ADW_PREFERENCES_ROW (row), ephy_password_record_get_origin (record));
+    adw_expander_row_set_subtitle (ADW_EXPANDER_ROW (row), ephy_password_record_get_username (record));
+    adw_expander_row_set_show_enable_switch (ADW_EXPANDER_ROW (row), FALSE);
 
-    button = gtk_button_new_from_icon_name ("edit-copy-symbolic", GTK_ICON_SIZE_BUTTON);
+    button = gtk_button_new_from_icon_name ("edit-copy-symbolic");
     gtk_widget_set_valign (button, GTK_ALIGN_CENTER);
     gtk_widget_set_tooltip_text (button, _("Copy password"));
-    hdy_expander_row_add_action (HDY_EXPANDER_ROW (row), button);
+    gtk_widget_add_css_class (button, "flat");
+    adw_expander_row_add_action (ADW_EXPANDER_ROW (row), button);
     g_signal_connect (button, "clicked", G_CALLBACK (copy_password_clicked), (void *)(ephy_password_record_get_password (record)));
 
     /* Username */
-    sub_row = hdy_action_row_new ();
-    hdy_preferences_row_set_title (HDY_PREFERENCES_ROW (sub_row), _("Username"));
-    gtk_container_add (GTK_CONTAINER (row), sub_row);
-
-    entry = gtk_entry_new ();
-    gtk_widget_set_hexpand (entry, TRUE);
-    gtk_widget_set_valign (entry, GTK_ALIGN_CENTER);
-    gtk_editable_set_editable (GTK_EDITABLE (entry), FALSE);
-    gtk_entry_set_alignment (GTK_ENTRY (entry), 1.0f);
-    gtk_entry_set_has_frame (GTK_ENTRY (entry), FALSE);
+    sub_row = adw_entry_row_new ();
+    adw_preferences_row_set_title (ADW_PREFERENCES_ROW (sub_row), _("Username"));
+    gtk_editable_set_editable (GTK_EDITABLE (sub_row), FALSE);
+    adw_expander_row_add_row (ADW_EXPANDER_ROW (row), sub_row);
 
     text = ephy_password_record_get_username (record);
     if (text)
-      gtk_entry_set_text (GTK_ENTRY (entry), text);
+      gtk_editable_set_text (GTK_EDITABLE (sub_row), text);
 
-    gtk_container_add (GTK_CONTAINER (sub_row), entry);
-
-    separator = gtk_separator_new (GTK_ORIENTATION_VERTICAL);
-    gtk_widget_set_margin_top (separator, 8);
-    gtk_widget_set_margin_bottom (separator, 8);
-    gtk_container_add (GTK_CONTAINER (sub_row), separator);
-
-    button = gtk_button_new_from_icon_name ("edit-copy-symbolic", GTK_ICON_SIZE_BUTTON);
+    button = gtk_button_new_from_icon_name ("edit-copy-symbolic");
     g_signal_connect (button, "clicked", G_CALLBACK (copy_username_clicked), (void *)(ephy_password_record_get_username (record)));
     gtk_widget_set_tooltip_text (button, _("Copy username"));
     gtk_widget_set_valign (button, GTK_ALIGN_CENTER);
-    gtk_container_add (GTK_CONTAINER (sub_row), button);
+    gtk_widget_add_css_class (button, "flat");
+    adw_entry_row_add_suffix (ADW_ENTRY_ROW (sub_row), button);
 
     /* Password */
-    sub_row = hdy_action_row_new ();
-    hdy_preferences_row_set_title (HDY_PREFERENCES_ROW (sub_row), _("Password"));
-    gtk_container_add (GTK_CONTAINER (row), sub_row);
-
-    entry = gtk_entry_new ();
-    gtk_widget_set_hexpand (entry, TRUE);
-    gtk_widget_set_valign (entry, GTK_ALIGN_CENTER);
-    gtk_editable_set_editable (GTK_EDITABLE (entry), FALSE);
-    gtk_entry_set_alignment (GTK_ENTRY (entry), 1.0f);
-    gtk_entry_set_has_frame (GTK_ENTRY (entry), FALSE);
-    gtk_entry_set_visibility (GTK_ENTRY (entry), FALSE);
+    sub_row = adw_password_entry_row_new ();
+    adw_preferences_row_set_title (ADW_PREFERENCES_ROW (sub_row), _("Password"));
+    gtk_editable_set_editable (GTK_EDITABLE (sub_row), FALSE);
+    adw_expander_row_add_row (ADW_EXPANDER_ROW (row), sub_row);
 
     text = ephy_password_record_get_password (record);
     if (text)
-      gtk_entry_set_text (GTK_ENTRY (entry), text);
-
-    gtk_container_add (GTK_CONTAINER (sub_row), entry);
-
-    separator = gtk_separator_new (GTK_ORIENTATION_VERTICAL);
-    gtk_widget_set_margin_top (separator, 8);
-    gtk_widget_set_margin_bottom (separator, 8);
-    gtk_container_add (GTK_CONTAINER (sub_row), separator);
-
-    button = gtk_toggle_button_new ();
-    image = gtk_image_new_from_icon_name ("dialog-password-symbolic", GTK_ICON_SIZE_BUTTON);
-    gtk_button_set_image (GTK_BUTTON (button), image);
-    gtk_widget_set_tooltip_text (button, _("Reveal password"));
-    gtk_widget_set_valign (button, GTK_ALIGN_CENTER);
-
-    g_object_bind_property (G_OBJECT (button), "active", G_OBJECT (entry), "visibility", G_BINDING_DEFAULT);
-    gtk_container_add (GTK_CONTAINER (sub_row), button);
+      gtk_editable_set_text (GTK_EDITABLE (sub_row), text);
 
     /* Remove button */
-    sub_row = hdy_action_row_new ();
-    gtk_container_add (GTK_CONTAINER (row), sub_row);
+    sub_row = adw_action_row_new ();
+    adw_expander_row_add_row (ADW_EXPANDER_ROW (row), sub_row);
 
     button = gtk_button_new_with_label (_("Remove Password"));
     gtk_widget_set_valign (button, GTK_ALIGN_CENTER);
-    dzl_gtk_widget_add_style_class (button, "destructive-action");
+    gtk_widget_add_css_class (button, "destructive-action");
     g_signal_connect (button, "clicked", G_CALLBACK (forget_clicked), record);
-    gtk_container_add (GTK_CONTAINER (sub_row), button);
+    adw_action_row_add_suffix (ADW_ACTION_ROW (sub_row), button);
 
     g_object_set_data (G_OBJECT (record), "passwords-view", passwords_view);
 
-    gtk_list_box_insert (GTK_LIST_BOX (passwords_view->listbox), row, -1);
+    gtk_list_box_append (GTK_LIST_BOX (passwords_view->listbox), row);
   }
 
-  if (g_list_length (records)) {
+  if (g_list_length (records))
     ephy_data_view_set_has_data (EPHY_DATA_VIEW (passwords_view), TRUE);
-    gtk_widget_show_all (passwords_view->listbox);
-  }
 
   g_assert (!passwords_view->records);
   passwords_view->records = g_list_copy_deep (records, (GCopyFunc)g_object_ref, NULL);
@@ -377,15 +318,6 @@ create_action_group (EphyPasswordsView *passwords_view)
   return G_ACTION_GROUP (group);
 }
 
-static void
-show_dialog_cb (GtkWidget *widget,
-                gpointer   user_data)
-{
-  EphyPasswordsView *passwords_view = EPHY_PASSWORDS_VIEW (widget);
-
-  populate_model (passwords_view);
-}
-
 static gboolean
 password_filter (GtkListBoxRow *row,
                  gpointer       user_data)
@@ -398,7 +330,7 @@ password_filter (GtkListBoxRow *row,
   const char *search_text = ephy_data_view_get_search_text (EPHY_DATA_VIEW (passwords_view));
 
   if (search_text == NULL) {
-    gtk_widget_show (GTK_WIDGET (row));
+    gtk_widget_set_visible (GTK_WIDGET (row), TRUE);
 
     return TRUE;
   }
@@ -431,8 +363,8 @@ ephy_passwords_view_init (EphyPasswordsView *passwords_view)
 
   passwords_view->cancellable = g_cancellable_new ();
 
-  g_signal_connect (GTK_WIDGET (passwords_view), "show", G_CALLBACK (show_dialog_cb), NULL);
-
   gtk_list_box_set_filter_func (GTK_LIST_BOX (passwords_view->listbox), password_filter, passwords_view, NULL);
   gtk_list_box_set_selection_mode (GTK_LIST_BOX (passwords_view->listbox), GTK_SELECTION_NONE);
+
+  populate_model (passwords_view);
 }
